/*
 *  Copyright (c) 2008-2012 Sonatype, Inc. All rights reserved.
 *
 *  This program is licensed to you under the Apache License Version 2.0,
 *  and you may not use this file except in compliance with the Apache License Version 2.0.
 *  You may obtain a copy of the Apache License Version 2.0 at http://www.apache.org/licenses/LICENSE-2.0.
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the Apache License Version 2.0 is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the Apache License Version 2.0 for the specific language governing permissions and limitations there under.
 */
package org.sonatype.jira.internal;

import java.lang.reflect.Method;

import javax.inject.Inject;

import org.apache.xmlrpc.XmlRpcException;
import org.apache.xmlrpc.XmlRpcHandler;
import org.apache.xmlrpc.XmlRpcRequest;
import org.apache.xmlrpc.server.AbstractReflectiveHandlerMapping;
import org.apache.xmlrpc.server.ReflectiveXmlRpcHandler;
import org.apache.xmlrpc.server.RequestProcessorFactoryFactory.RequestProcessorFactory;
import org.sonatype.jira.JiraXmlRpc;

/**
 * Mapping class for JIRA XMLRPC handlers.
 */
public class JIRAHandlerMapping
    extends AbstractReflectiveHandlerMapping
{

    private final Class<? extends JiraXmlRpc> handlerClass;

    private JiraXmlRpc handler;

    public JIRAHandlerMapping( final Class<? extends JiraXmlRpc> handlerClass )
        throws XmlRpcException
    {
        registerPublicMethods( "jira1", handlerClass );
        this.handlerClass = handlerClass;
    }

    @Inject
    public JIRAHandlerMapping( final JiraXmlRpc handler )
        throws XmlRpcException
    {
        this.handler = handler;
        this.handlerClass = handler.getClass();
        registerPublicMethods( "jira1", handlerClass );
    }

    @Override
    protected XmlRpcHandler newXmlRpcHandler( @SuppressWarnings( "rawtypes" ) final Class pClass,
                                              final Method[] pMethods )
        throws XmlRpcException
    {
        if ( this.handler != null )
        {
            RequestProcessorFactory factory = getRequestProcessorFactoryFactory().getRequestProcessorFactory( pClass );
            factory = new RequestProcessorFactory()
            {
                @Override
                public Object getRequestProcessor( final XmlRpcRequest pRequest )
                    throws XmlRpcException
                {
                    return handler;
                }
            };
            return new ReflectiveXmlRpcHandler( this, getTypeConverterFactory(), pClass, factory, pMethods);
        }
        else
        {
            return super.newXmlRpcHandler( pClass, pMethods );
        }
    }

    @Override
    protected boolean isHandlerMethod( final Method pMethod )
    {
        return super.isHandlerMethod( pMethod ) && ( !pMethod.getName().equals( "invoke" ) );
    }

}
