/*
 *  Copyright (c) 2008-2012 Sonatype, Inc. All rights reserved.
 *
 *  This program is licensed to you under the Apache License Version 2.0,
 *  and you may not use this file except in compliance with the Apache License Version 2.0.
 *  You may obtain a copy of the Apache License Version 2.0 at http://www.apache.org/licenses/LICENSE-2.0.
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the Apache License Version 2.0 is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the Apache License Version 2.0 for the specific language governing permissions and limitations there under.
 */
package org.sonatype.jira.recorder;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.lang.reflect.Method;
import java.util.List;

import com.thoughtworks.xstream.XStream;

public class Recorder
{

    private static XStream XSTREAM;

    static
    {
        XSTREAM = new XStream();
        XSTREAM.alias( "method", Method.class );
        XSTREAM.alias( "call", Triple.class );
    }

    public static void write( List<Triple> triplets, File out )
        throws IOException
    {
        FileOutputStream fos = null;
        try
        {
            fos = new FileOutputStream( out );

            XSTREAM.toXML( triplets, fos );
        }
        finally
        {
            if ( fos != null )
            {
                fos.close();
            }
        }
    }

    @SuppressWarnings( "unchecked" )
    public static List<Triple> read( File in )
        throws IOException
    {
        FileInputStream fis = null;
        try
        {
            fis = new FileInputStream( in );

            return (List<Triple>) XSTREAM.fromXML( fis );
        }
        finally
        {
            if ( fis != null )
            {
                fis.close();
            }
        }

    }
}
